/* array loader (make-array helper) */
static stt_obj_t *arrayloader(stt_varctx_t *ctx, stt_obj_t *ar, stt_obj_t **addr, int dim, stt_obj_t *list)
{
	stt_obj_t *val;
	long ds, i;

	ds = 1; /* compute offset */
	for(i=0; i<dim; i++)
		ds *= ar->d.array.size[i];

	if(dim == ar->d.array.dim-1) {
		/* final level: load elements */

		for(i=0; i<ar->d.array.size[dim]; i++) {
			if(!stt_nextparm(&val, &list))
				return stt_create_error(ctx->stt, STT_SE_PARM, "Initializer comes short");

			if(val->type == STT_ST_ERROR) return val;
			*addr = val;

			addr += ds;
		}
	} else {
		/* there are more dimensions: recursion */

		for(i=0; i<ar->d.array.size[dim]; i++) {
			if(!stt_nextparm(&val, &list))
				return stt_create_error(ctx->stt, STT_SE_PARM, "Initializer comes short");
			if(val->type != STT_ST_CONS)
				return stt_create_error(ctx->stt, STT_SE_TYPE, "Initializer must be a list");
			val = arrayloader(ctx, ar, addr, dim + 1, val);
			if(val->type == STT_ST_ERROR) return val;

			addr += ds;
		}
	}

	return ar;
}

static STT_BUILTIN(sb_make_array)
{
	stt_obj_t *pl, *ar, *obj;
	long size[STT_ARRAY_MAXDIMS];
	int dim = 0;

	STT_ARG_CONS(ctx->stt, obj);

	pl = obj;
	while(stt_nextparm(&obj, &pl)) {
		if(dim == STT_ARRAY_MAXDIMS)
			return stt_create_error(ctx->stt, STT_SE_PARM, "Array has too many dimensions");

		if(obj->type != STT_ST_INT)
			return stt_create_error(ctx->stt, STT_SE_TYPE, "Array dimensions must be integers");
		if(obj->d.integer.value < 1)
			return stt_create_error(ctx->stt, STT_SE_PARM, "Array dimension must be >= 1");

		size[dim++] = obj->d.integer.value;
	}

	if(!dim)
		return stt_create_error(ctx->stt, STT_SE_PARM, "Can't create zero-dimensional array");

	ar = stt_create_array(ctx->stt, dim, size);
	if(ar->type == STT_ST_ERROR) return ar; /* create_array might throw STT_SE_MEM */

	if(stt_nextarg(ctx, &obj, &parm)) {
		/* initializer present */

		if(obj->type == STT_ST_ERROR) return obj;
		if(obj->type != STT_ST_CONS)
			return stt_create_error(ctx->stt, STT_SE_TYPE, "Initializer must be a list");

		obj = arrayloader(ctx, ar, ar->d.array.data, 0, obj);
		if(obj->type == STT_ST_ERROR) return obj;
	}

	return ar;
}

static STT_BUILTIN(sb_adim)
{
	stt_obj_t *l = NULL, *lt = NULL, *ar;
	int i;

	STT_ARG_ARRAY(ctx->stt, ar);

	for(i=0; i<ar->d.array.dim; i++) {
		if(!lt) l = lt = stt_create_cons(ctx->stt, stt_create_int(ctx->stt, ar->d.array.size[i]), stt_create_nil(ctx->stt));
		else lt = lt->d.cons.cdr = stt_create_cons(ctx->stt, stt_create_int(ctx->stt, ar->d.array.size[i]), stt_create_nil(ctx->stt));
	}

	return l;
}

static STT_BUILTIN(sb_aset)
{
	stt_obj_t *obj, *val, *ar;
	stt_obj_t **addr;
	long ds = 1;
	int i;

	STT_ARG_ARRAY(ctx->stt, ar);

	addr = ar->d.array.data;
	for(i=0; i<ar->d.array.dim; i++) {
		if(!stt_nextarg(ctx, &obj, &parm))
			return stt_create_error(ctx->stt, STT_SE_PNUM, "Insufficient array indices for aset");
		if(obj->type == STT_ST_ERROR) return obj;
		if(obj->type != STT_ST_INT)
			return stt_create_error(ctx->stt, STT_SE_TYPE, "aset index must be an integer");
		if(obj->d.integer.value < 0 || obj->d.integer.value >= ar->d.array.size[i])
			return stt_create_error(ctx->stt, STT_SE_PNUM, "aset index out of bounds");
		addr += (ds * obj->d.integer.value);
		ds *= ar->d.array.size[i];
	}

	if(!stt_nextarg(ctx, &val, &parm))
		return stt_create_error(ctx->stt, STT_SE_PNUM, "Insufficient parameters for aset");
	if(val->type == STT_ST_ERROR) return val;

	*addr = val;

	if((stt_intptr_t)data == 1) {
		/* weak-aset */
		stt_weak_check(val);
		stt_weak_reference(ar, ar->serial, addr);
	}

	return val;
}

static STT_BUILTIN(sb_aref)
{
	stt_obj_t *obj, *ar;
	stt_obj_t **addr;
	long ds = 1;
	int i;

	STT_ARG_ARRAY(ctx->stt, ar);

	addr = ar->d.array.data;
	for(i=0; i<ar->d.array.dim; i++) {
		if(!stt_nextarg(ctx, &obj, &parm))
			return stt_create_error(ctx->stt, STT_SE_PNUM, "Insufficient array indices for aref");
		if(obj->type == STT_ST_ERROR) return obj;
		if(obj->type != STT_ST_INT)
			return stt_create_error(ctx->stt, STT_SE_TYPE, "aref index must be an integer");
		if(obj->d.integer.value < 0 || obj->d.integer.value >= ar->d.array.size[i])
			return stt_create_error(ctx->stt, STT_SE_PNUM, "aref index out of bounds");
		addr += (ds * obj->d.integer.value);
		ds *= ar->d.array.size[i];
	}

	if(!*addr) return stt_create_nil(ctx->stt);
	return *addr;
}
