static STT_BUILTIN(sb_get_time)
{
	return stt_create_int(ctx->stt, time(NULL));
}

static STT_BUILTIN(sb_mktime)
{
	stt_obj_t *year, *month, *day, *hours, *minutes, *seconds;
	struct tm mtm;

	STT_ARG_INT(ctx->stt, year);
	STT_ARG_INT(ctx->stt, month);
	STT_ARG_INT(ctx->stt, day);
	STT_ARG_INT(ctx->stt, hours);
	STT_ARG_INT(ctx->stt, minutes);
	STT_ARG_INT(ctx->stt, seconds);

	mtm.tm_year = year->d.integer.value - 1900;
	mtm.tm_mon = month->d.integer.value - 1;
	mtm.tm_mday = day->d.integer.value;
	mtm.tm_hour = hours->d.integer.value;
	mtm.tm_min = minutes->d.integer.value;
	mtm.tm_sec = seconds->d.integer.value;
    mtm.tm_isdst = -1;
	return stt_create_int(ctx->stt, mktime(&mtm));
}

static STT_BUILTIN(sb_get_decoded_time)
{
	stt_obj_t *obj;
	time_t t;
	struct tm *gm;
#ifndef timezone
	int timezone = 0;
#endif

	int gmt = (stt_intptr_t) data;

	if(stt_nextarg(ctx, &obj, &parm)) {
		if(obj->type == STT_ST_ERROR) return obj;
		if(obj->type != STT_ST_INT)
			return stt_create_error(ctx->stt, STT_SE_TYPE, "Parameter of get-decoded-time must be an integer");
		t = obj->d.integer.value;
	} else t = time(NULL);

	if(gmt)
		gm = gmtime(&t);
	else
		gm = localtime(&t);

	return stt_create_cons(ctx->stt, stt_create_int(ctx->stt, gm->tm_sec),
			stt_create_cons(ctx->stt, stt_create_int(ctx->stt, gm->tm_min),
			stt_create_cons(ctx->stt, stt_create_int(ctx->stt, gm->tm_hour),
			stt_create_cons(ctx->stt, stt_create_int(ctx->stt, gm->tm_mday),
			stt_create_cons(ctx->stt, stt_create_int(ctx->stt, gm->tm_mon + 1),
			stt_create_cons(ctx->stt, stt_create_int(ctx->stt, gm->tm_year + 1900),
			stt_create_cons(ctx->stt, stt_create_int(ctx->stt, gm->tm_wday == 0 ? 6 : gm->tm_wday - 1),
			stt_create_cons(ctx->stt, gm->tm_isdst ? stt_create_t(ctx->stt) : stt_create_nil(ctx->stt),
			stt_create_cons(ctx->stt, stt_create_int(ctx->stt, -timezone / 3600), stt_create_nil(ctx->stt))))))))));
}

static STT_BUILTIN(sb_sleep)
{
	stt_obj_t *tim;
	STT_ARG_INT(ctx->stt, tim);

#ifndef WIN32
	sleep(tim->d.integer.value);
#else
	Sleep(tim->d.integer.value * 1000);
#endif

	return tim;
}

